@@ -14,12 +14,16 @@ import math
 import warnings
 import functools
 
+try:
+	import collections.abc
+except ImportError:
+	collections.abc = collections
+
 import six
 from six.moves import queue, xrange as range
 
 
-from more_itertools import more
-from more_itertools import recipes
+import more_itertools
 
 
 def make_rows(num_columns, seq):
@@ -37,7 +41,11 @@ def make_rows(num_columns, seq):
 	if partial:
 		num_rows += 1
 	# break the seq into num_columns of length num_rows
-	result = recipes.grouper(num_rows, seq)
+	try:
+		result = more_itertools.grouper(seq, num_rows)
+	except TypeError:
+		# more_itertools before 6.x
+		result = more_itertools.grouper(num_rows, seq)
 	# result is now a list of columns... transpose it to return a list
 	# of rows
 	return zip(*result)
@@ -387,7 +395,7 @@ def grouper_nofill_str(n, iterable):
 	>>> tuple(grouper_nofill_str(3, list(range(10))))
 	([0, 1, 2], [3, 4, 5], [6, 7, 8], [9])
 	"""
-	res = more.chunked(iterable, n)
+	res = more_itertools.chunked(iterable, n)
 	if isinstance(iterable, six.string_types):
 		res = (''.join(item) for item in res)
 	return res
@@ -438,30 +446,21 @@ class Counter(object):
 	next = __next__
 
 	def GetCount(self):
-		warnings.warn("Use count attribute directly", DeprecationWarning)
+		warnings.warn(
+			"Use count attribute directly", DeprecationWarning,
+			stacklevel=2)
 		return self.count
 
 # todo, factor out caching capability
 
 
 class iterable_test(dict):
-	"""
-	*Deprecated*
-
-	Test objects for iterability, caching the result by type
-
-	>>> test = iterable_test()
-	>>> test['foo']
-	False
-	>>> test[[]]
-	True
-	"""
-
 	def __init__(self, ignore_classes=six.string_types + (six.binary_type,)):
 		"""ignore_classes must include str, because if a string
 		is iterable, so is a single character, and the routine runs
 		into an infinite recursion"""
-		warnings.warn("Slated for removal", DeprecationWarning)
+		warnings.warn(
+			"Slated for removal", DeprecationWarning, stacklevel=2)
 		assert set(six.string_types) <= set(ignore_classes), (
 			'str must be in ignore_classes')
 		self.ignore_classes = ignore_classes
@@ -495,27 +494,12 @@ def iflatten(subject, test=None):
 def flatten(subject, test=None):
 	"""
 	*Deprecated*: Use more_itertools.collapse instead.
-
-	Flatten an iterable with possible nested iterables.
-
-	Adapted from
-	http://mail.python.org/pipermail/python-list/2003-November/233971.html
-
-	>>> flatten(['a','b',['c','d',['e','f'],'g'],'h'])
-	['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']
-
-	Note this will normally ignore string types as iterables.
-
-	>>> flatten(['ab', 'c'])
-	['ab', 'c']
-
-	Same for bytes
-
-	>>> flatten([b'ab', b'c'])
-	[b'ab', b'c']
 	"""
-	warnings.warn("Use more_itertools.collapse instead", DeprecationWarning)
-	return list(more.collapse(subject, base_type=(bytes,)))
+	warnings.warn(
+		"Use more_itertools.collapse instead",
+		DeprecationWarning,
+		stacklevel=2)
+	return list(more_itertools.collapse(subject, base_type=(bytes,)))
 
 
 def empty():
@@ -711,11 +695,32 @@ def takewhile_peek(predicate, iterable):
 
 	>>> list(items)
 	[4, 5, 6, 7, 8, 9]
+
+	>>> empty = takewhile_peek(is_small, Peekable([]))
+	>>> list(empty)
+	[]
+
+	>>> items = Peekable([3])
+	>>> small_items = takewhile_peek(is_small, items)
+	>>> list(small_items)
+	[3]
+	>>> list(items)
+	[]
+
+	>>> items = Peekable([4])
+	>>> small_items = takewhile_peek(is_small, items)
+	>>> list(small_items)
+	[]
+	>>> list(items)
+	[4]
 	"""
 	while True:
-		if not predicate(iterable.peek()):
+		try:
+			if not predicate(iterable.peek()):
+				break
+			yield next(iterable)
+		except StopIteration:
 			break
-		yield next(iterable)
 
 
 def first(iterable, *args):
@@ -916,14 +921,11 @@ def always_iterable(item):
 	Although mappings are iterable, treat each like a singleton, as
 	it's more like an object than a sequence.
 
-	>>> always_iterable(dict(a=1))
-	({'a': 1},)
+	>>> next(always_iterable(dict(a=1)))
+	{'a': 1}
 	"""
-	return (
-		(item,)
-		if isinstance(item, collections.Mapping)
-		else more.always_iterable(item)
-	)
+	base_types = six.text_type, bytes, collections.abc.Mapping
+	return more_itertools.always_iterable(item, base_type=base_types)
 
 
 def suppress_exceptions(callables, *exceptions):
@@ -974,13 +976,8 @@ def list_or_single(iterable):
 	"""
 	Given an iterable, return the items as a list. If the iterable contains
 	exactly one item, return that item. Correlary function to always_iterable.
-
-	>>> list_or_single(iter('abcd'))
-	['a', 'b', 'c', 'd']
-	>>> list_or_single(['a'])
-	'a'
 	"""
-	warnings.warn("Use maybe_single", DeprecationWarning)
+	warnings.warn("Use maybe_single", DeprecationWarning, stacklevel=2)
 	return maybe_single(list(iterable))
 
 
@@ -1010,3 +1007,246 @@ def self_product(iterable):
 	[(1, 1), (1, 2), ..., (3, 3)]
 	"""
 	return itertools.product(*itertools.tee(iterable))
+
+
+def duplicates(*iterables, **kwargs):
+	"""
+	Yield duplicate items from any number of sorted iterables of items
+
+	>>> items_a = [1, 2, 3]
+	>>> items_b = [0, 3, 4, 5, 6]
+	>>> list(duplicates(items_a, items_b))
+	[(3, 3)]
+
+	It won't behave as you expect if the iterables aren't ordered
+
+	>>> items_b.append(1)
+	>>> list(duplicates(items_a, items_b))
+	[(3, 3)]
+	>>> list(duplicates(items_a, sorted(items_b)))
+	[(1, 1), (3, 3)]
+
+	This function is most interesting when it's operating on a key
+	of more complex objects.
+
+	>>> items_a = [dict(email='joe@example.com', id=1)]
+	>>> items_b = [dict(email='joe@example.com', id=2), dict(email='other')]
+	>>> dupe, = duplicates(items_a, items_b, key=operator.itemgetter('email'))
+	>>> dupe[0]['email'] == dupe[1]['email'] == 'joe@example.com'
+	True
+	>>> dupe[0]['id']
+	1
+	>>> dupe[1]['id']
+	2
+	"""
+	key = kwargs.pop('key', lambda x: x)
+	assert not kwargs
+	zipped = more_itertools.collate(*iterables, key=key)
+	grouped = itertools.groupby(zipped, key=key)
+	groups = (
+		tuple(g)
+		for k, g in grouped
+	)
+
+	def has_dupes(group):
+		return len(group) > 1
+	return filter(has_dupes, groups)
+
+
+def assert_ordered(iterable, key=lambda x: x, comp=operator.le):
+	"""
+	Assert that for all items in the iterable, they're in order based on comp
+
+	>>> list(assert_ordered(range(5)))
+	[0, 1, 2, 3, 4]
+	>>> list(assert_ordered(range(5), comp=operator.ge))
+	Traceback (most recent call last):
+	...
+	AssertionError: 0 < 1
+	>>> list(assert_ordered(range(5, 0, -1), key=operator.neg))
+	[5, 4, 3, 2, 1]
+	"""
+	err_tmpl = (
+		"{pair[0]} > {pair[1]}" if comp is operator.le else
+		"{pair[0]} < {pair[1]}" if comp is operator.ge else
+		"not {comp} {pair}"
+	)
+	for pair in more_itertools.pairwise(iterable):
+		keyed = tuple(map(key, pair))
+		assert comp(*keyed), err_tmpl.format(**locals())
+		yield pair[0]
+	yield pair[1]
+
+
+def collate_revs(old, new, key=lambda x: x, merge=lambda old, new: new):
+	"""
+	Given revision sets old and new, each containing a series
+	of revisions of some set of objects, collate them based on
+	these rules:
+
+	- all items from each set are yielded in stable order
+	- items in old are yielded first
+	- items in new are yielded last
+	- items that match are yielded in the order in which they
+	appear, giving preference to new
+
+	Items match based on the 'key' parameter (identity by default).
+
+	Items are merged using the 'merge' function, which accepts the old
+	and new items to be merged (returning new by default).
+
+	This algorithm requires fully materializing both old and new in memory.
+
+	>>> rev1 = ['a', 'b', 'c']
+	>>> rev2 = ['a', 'd', 'c']
+	>>> result = list(collate_revs(rev1, rev2))
+
+	'd' must appear before 'c'
+	>>> result.index('d') < result.index('c')
+	True
+
+	'b' must appear before 'd' because it came chronologically
+	first.
+	>>> result.index('b') < result.index('d')
+	True
+
+	>>> result
+	['a', 'b', 'd', 'c']
+
+	>>> list(collate_revs(['a', 'b', 'c'], ['d']))
+	['a', 'b', 'c', 'd']
+
+	>>> list(collate_revs(['b', 'a'], ['a', 'b']))
+	['a', 'b']
+
+	>>> list(collate_revs(['a', 'c'], ['a', 'b', 'c']))
+	['a', 'b', 'c']
+
+	Given two sequences of things out of order, regardless
+	of which order in which the items are merged, all
+	keys should always be merged.
+
+	>>> from more_itertools import consume
+	>>> left_items = ['a', 'b', 'c']
+	>>> right_items = ['a', 'c', 'b']
+	>>> consume(collate_revs(left_items, right_items, merge=print))
+	a a
+	c c
+	b b
+	>>> consume(collate_revs(right_items, left_items, merge=print))
+	a a
+	b b
+	c c
+
+	The merge should not suppress non-True items:
+
+	>>> consume(collate_revs([0, 1, 2, None, ''], [0, None, ''], merge=print))
+	None None
+	<BLANKLINE>
+	0 0
+
+	"""
+	missing = object()
+
+	def maybe_merge(*items):
+		"""
+		Merge any non-null items
+		"""
+		def not_missing(ob):
+			return ob is not missing
+
+		return functools.reduce(merge, filter(not_missing, items))
+
+	new_items = collections.OrderedDict(
+		(key(el), el)
+		for el in new
+	)
+	old_items = collections.OrderedDict(
+		(key(el), el)
+		for el in old
+	)
+
+	# use the old_items as a reference
+	for old_key, old_item in _mutable_iter(old_items):
+		if old_key not in new_items:
+			yield old_item
+			continue
+
+		# yield all new items that appear before the matching key
+		before, match_new, new_items = _swap_on_miss(
+			partition_dict(new_items, old_key))
+		for new_key, new_item in before.items():
+			# ensure any new keys are merged with previous items if
+			# they exist
+			yield maybe_merge(new_item, old_items.pop(new_key, missing))
+		yield merge(old_item, match_new)
+
+	# finally, yield whatever is leftover
+	# yield from new_items.values()
+	for item in new_items.values():
+		yield item
+
+
+def _mutable_iter(dict):
+	"""
+	Iterate over items in the dict, yielding the first one, but allowing
+	it to be mutated during the process.
+	>>> d = dict(a=1)
+	>>> it = _mutable_iter(d)
+	>>> next(it)
+	('a', 1)
+	>>> d
+	{}
+	>>> d.update(b=2)
+	>>> list(it)
+	[('b', 2)]
+	"""
+	while dict:
+		prev_key = next(iter(dict))
+		yield prev_key, dict.pop(prev_key)
+
+
+def _swap_on_miss(partition_result):
+	"""
+	Given a partition_dict result, if the partition missed, swap
+	the before and after.
+	"""
+	before, item, after = partition_result
+	return (before, item, after) if item else (after, item, before)
+
+
+def partition_dict(items, key):
+	"""
+	Given an ordered dictionary of items and a key in that dict,
+	return an ordered dict of items before, the keyed item, and
+	an ordered dict of items after.
+
+	>>> od = collections.OrderedDict(zip(range(5), 'abcde'))
+	>>> before, item, after = partition_dict(od, 3)
+	>>> before
+	OrderedDict([(0, 'a'), (1, 'b'), (2, 'c')])
+	>>> item
+	'd'
+	>>> after
+	OrderedDict([(4, 'e')])
+
+	Like string.partition, if the key is not found in the items,
+	the before will contain all items, item will be None, and
+	after will be an empty iterable.
+
+	>>> before, item, after = partition_dict(od, -1)
+	>>> before
+	OrderedDict([(0, 'a'), ..., (4, 'e')])
+	>>> item
+	>>> list(after)
+	[]
+	"""
+	def unmatched(pair):
+		test_key, item, = pair
+		return test_key != key
+
+	items_iter = iter(items.items())
+	item = items.get(key)
+	left = collections.OrderedDict(itertools.takewhile(unmatched, items_iter))
+	right = collections.OrderedDict(items_iter)
+	return left, item, right
